<?php
/*
Plugin Name: SEO Tag Generator
Description: A plugin to generate SEO-friendly tags from post content with advanced options.
Version: 1.7
Author: Developer Tyga Sparta
Author URI: https://tyga-sparta.com
License: GPLv2 or later
License URI: https://www.gnu.org/licenses/gpl-2.0.html
*/

// Exit if accessed directly
if (!defined('ABSPATH')) {
    exit;
}

// Enable auto-updates for this plugin
add_filter('auto_update_plugin', 'stg_enable_auto_updates', 10, 2);
function stg_enable_auto_updates($update, $plugin) {
    // Check if this is your plugin
    if ($plugin == plugin_basename(__FILE__)) {
        return true; // Enable auto-updates
    }
    return $update; // Leave other plugins unchanged
}

// Enqueue custom admin styles
function stg_enqueue_admin_styles($hook) {
    // Load only on the plugin's settings page
    if ($hook != 'settings_page_seo-tag-generator-settings') {
        return;
    }
    // Enqueue the CSS file
    wp_enqueue_style('stg-admin-styles', plugins_url('admin-styles.css', __FILE__));
}
add_action('admin_enqueue_scripts', 'stg_enqueue_admin_styles');

// Function to extract keywords from post content
function stg_extract_keywords($content, $exclude_words = array()) {
    // Remove HTML tags
    $content = wp_strip_all_tags($content);

    // Convert to lowercase
    $content = strtolower($content);

    // Remove stopwords (you can expand this list)
    $stopwords = array('the', 'and', 'in', 'of', 'to', 'a', 'is', 'it', 'that', 'this', 'with', 'for', 'on', 'are', 'as', 'be', 'by', 'or', 'was', 'so', 'but', 'not', 'have', 'has', 'had', 'at', 'from', 'an', 'which', 'you', 'we', 'they', 'he', 'she', 'it', 'his', 'her', 'their', 'our', 'my', 'your', 'its', 'what', 'who', 'where', 'when', 'why', 'how', 'if', 'else', 'then', 'there', 'here', 'out', 'up', 'down', 'over', 'under', 'again', 'once', 'more', 'such', 'no', 'nor', 'only', 'own', 'same', 'so', 'than', 'too', 'very', 's', 't', 'can', 'will', 'just', 'don', 'should', 'now');

    // Merge stopwords with user-defined exclude words
    $exclude_words = array_merge($stopwords, $exclude_words);

    // Split content into words
    $words = explode(' ', $content);
    $keywords = array();

    // Filter out stopwords and short words
    foreach ($words as $word) {
        $word = preg_replace('/[^a-zA-Z0-9]/', '', $word); // Remove special characters
        if (!in_array($word, $exclude_words) && strlen($word) > 3) {
            $keywords[] = $word;
        }
    }

    // Count keyword frequency
    $keyword_count = array_count_values($keywords);

    // Sort by frequency
    arsort($keyword_count);

    return $keyword_count;
}

// Function to suggest tags based on extracted keywords
function stg_suggest_tags($post_id, $num_tags = 5, $exclude_words = array()) {
    $post = get_post($post_id);
    $content = $post->post_content;

    $keywords = stg_extract_keywords($content, $exclude_words);

    $suggested_tags = array();
    foreach ($keywords as $keyword => $count) {
        $suggested_tags[] = $keyword;
    }

    // Return the specified number of tags
    return array_slice($suggested_tags, 0, $num_tags);
}

// Add meta box to post editor
function stg_add_meta_box() {
    add_meta_box(
        'stg_suggested_tags', // Meta box ID
        'Suggested SEO Tags', // Meta box title
        'stg_render_meta_box', // Callback function to render meta box
        'post', // Post type
        'side', // Context (side, normal, advanced)
        'default' // Priority
    );
}
add_action('add_meta_boxes', 'stg_add_meta_box');

// Render the meta box content
function stg_render_meta_box($post) {
    // Get plugin options
    $num_tags = get_option('stg_num_tags', 5);
    $exclude_words = get_option('stg_exclude_words', '');

    // Convert exclude words to an array
    $exclude_words = !empty($exclude_words) ? explode(',', $exclude_words) : array();

    // Get suggested tags
    $suggested_tags = stg_suggest_tags($post->ID, $num_tags, $exclude_words);

    // Display suggested tags
    if (!empty($suggested_tags)) {
        echo '<p>Suggested Tags:</p>';
        echo '<ul class="stg-tag-list">';
        foreach ($suggested_tags as $tag) {
            echo '<li>' . esc_html($tag) . '</li>';
        }
        echo '</ul>';
    } else {
        echo '<p>No tags suggested.</p>';
    }

    // Add a link to the settings page
    echo '<p><a href="' . admin_url('options-general.php?page=seo-tag-generator-settings') . '" class="stg-settings-link">Configure Plugin Settings</a></p>';
}

// Auto-insert suggested tags when saving a post
function stg_auto_insert_tags($post_id) {
    // Check if this is a post and not an autosave
    if (wp_is_post_autosave($post_id) || wp_is_post_revision($post_id)) {
        return;
    }

    // Get plugin options
    $num_tags = get_option('stg_num_tags', 5);
    $exclude_words = get_option('stg_exclude_words', '');

    // Convert exclude words to an array
    $exclude_words = !empty($exclude_words) ? explode(',', $exclude_words) : array();

    // Get suggested tags
    $suggested_tags = stg_suggest_tags($post_id, $num_tags, $exclude_words);

    // Insert tags into the post
    if (!empty($suggested_tags)) {
        wp_set_post_tags($post_id, $suggested_tags, true);
    }
}
add_action('save_post', 'stg_auto_insert_tags');

// Add settings page
function stg_add_settings_page() {
    add_options_page(
        'SEO Tag Generator Settings', // Page title
        'SEO Tag Generator', // Menu title
        'manage_options', // Capability
        'seo-tag-generator-settings', // Menu slug
        'stg_render_settings_page' // Callback function
    );
}
add_action('admin_menu', 'stg_add_settings_page');

// Render the settings page
function stg_render_settings_page() {
    // Get excluded words
    $exclude_words = get_option('stg_exclude_words', '');
    $exclude_words_list = !empty($exclude_words) ? explode(',', $exclude_words) : array();

    ?>
    <div class="wrap stg-settings-wrap">
        <h1 class="stg-settings-title">SEO Tag Generator Settings</h1>
        <form method="post" action="options.php" class="stg-settings-form">
            <?php
            settings_fields('stg_options_group');
            do_settings_sections('seo-tag-generator-settings');
            submit_button('Save Settings', 'primary', 'submit', false);
            ?>
        </form>

        <!-- Display Excluded Words -->
        <div class="stg-excluded-words">
            <h2>Excluded Words</h2>
            <?php if (!empty($exclude_words_list)) : ?>
                <ul class="stg-excluded-words-list">
                    <?php foreach ($exclude_words_list as $word) : ?>
                        <li><?php echo esc_html(trim($word)); ?></li>
                    <?php endforeach; ?>
                </ul>
            <?php else : ?>
                <p>No words are currently excluded.</p>
            <?php endif; ?>
        </div>

        <!-- Developer Credit -->
        <div class="stg-credit">
            <p>Developed by <a href="https://tyga-sparta.com" target="_blank">Tyga Sparta</a></p>
        </div>
    </div>
    <?php
}

// Register plugin settings
function stg_register_settings() {
    register_setting('stg_options_group', 'stg_num_tags', 'intval');
    register_setting('stg_options_group', 'stg_exclude_words', 'sanitize_text_field');

    add_settings_section(
        'stg_main_section',
        'Main Settings',
        null,
        'seo-tag-generator-settings'
    );

    add_settings_field(
        'stg_num_tags',
        'Number of Tags to Generate',
        'stg_num_tags_callback',
        'seo-tag-generator-settings',
        'stg_main_section'
    );

    add_settings_field(
        'stg_exclude_words',
        'Exclude Words (comma-separated)',
        'stg_exclude_words_callback',
        'seo-tag-generator-settings',
        'stg_main_section'
    );
}
add_action('admin_init', 'stg_register_settings');

// Callback for number of tags field
function stg_num_tags_callback() {
    $num_tags = get_option('stg_num_tags', 5);
    echo '<input type="number" name="stg_num_tags" value="' . esc_attr($num_tags) . '" min="1" max="20" class="stg-input" />';
}

// Callback for exclude words field
function stg_exclude_words_callback() {
    $exclude_words = get_option('stg_exclude_words', '');
    echo '<input type="text" name="stg_exclude_words" value="' . esc_attr($exclude_words) . '" placeholder="e.g., word1,word2,word3" class="stg-input" />';
}

// Activation hook
function stg_activate() {
    // Set default options
    add_option('stg_num_tags', 5);
    add_option('stg_exclude_words', '');
}
register_activation_hook(__FILE__, 'stg_activate');

// Deactivation hook
function stg_deactivate() {
    // Deactivation code if needed
}
register_deactivation_hook(__FILE__, 'stg_deactivate');